# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/trouver',
                'doc_host': 'https://hyunjongkimmath.github.io',
                'git_url': 'https://github.com/hyunjongkimmath/trouver',
                'lib_path': 'trouver'},
  'syms': { 'trouver.core': {'trouver.core.foo': ('core.html#foo', 'trouver/core.py')},
            'trouver.helper': { 'trouver.helper._str_parts': ('helper.html#_str_parts', 'trouver/helper.py'),
                                'trouver.helper._test_directory': ('helper.html#_test_directory', 'trouver/helper.py'),
                                'trouver.helper.alphabet_or_latex_command_to_alphabet': ( 'helper.html#alphabet_or_latex_command_to_alphabet',
                                                                                          'trouver/helper.py'),
                                'trouver.helper.alphabet_or_latex_command_to_alphabet_group': ( 'helper.html#alphabet_or_latex_command_to_alphabet_group',
                                                                                                'trouver/helper.py'),
                                'trouver.helper.alphabet_to_alphabet_group': ( 'helper.html#alphabet_to_alphabet_group',
                                                                               'trouver/helper.py'),
                                'trouver.helper.containing_string_priority': ( 'helper.html#containing_string_priority',
                                                                               'trouver/helper.py'),
                                'trouver.helper.current_time_formatted_to_minutes': ( 'helper.html#current_time_formatted_to_minutes',
                                                                                      'trouver/helper.py'),
                                'trouver.helper.default_str_comparison': ('helper.html#default_str_comparison', 'trouver/helper.py'),
                                'trouver.helper.definition_asterisk_indices': ( 'helper.html#definition_asterisk_indices',
                                                                                'trouver/helper.py'),
                                'trouver.helper.defs_and_notats_separations': ( 'helper.html#defs_and_notats_separations',
                                                                                'trouver/helper.py'),
                                'trouver.helper.dict_with_keys_topologically_sorted': ( 'helper.html#dict_with_keys_topologically_sorted',
                                                                                        'trouver/helper.py'),
                                'trouver.helper.double_asterisk_indices': ('helper.html#double_asterisk_indices', 'trouver/helper.py'),
                                'trouver.helper.existing_path': ('helper.html#existing_path', 'trouver/helper.py'),
                                'trouver.helper.file_existence_test': ('helper.html#file_existence_test', 'trouver/helper.py'),
                                'trouver.helper.files_of_format_sorted': ('helper.html#files_of_format_sorted', 'trouver/helper.py'),
                                'trouver.helper.find_regex_in_text': ('helper.html#find_regex_in_text', 'trouver/helper.py'),
                                'trouver.helper.graph_for_topological_sort': ( 'helper.html#graph_for_topological_sort',
                                                                               'trouver/helper.py'),
                                'trouver.helper.is_number': ('helper.html#is_number', 'trouver/helper.py'),
                                'trouver.helper.latex_indices': ('helper.html#latex_indices', 'trouver/helper.py'),
                                'trouver.helper.latex_to_path_accepted_string': ( 'helper.html#latex_to_path_accepted_string',
                                                                                  'trouver/helper.py'),
                                'trouver.helper.natsort_comparison': ('helper.html#natsort_comparison', 'trouver/helper.py'),
                                'trouver.helper.notation_asterisk_indices': ('helper.html#notation_asterisk_indices', 'trouver/helper.py'),
                                'trouver.helper.path_name_no_ext': ('helper.html#path_name_no_ext', 'trouver/helper.py'),
                                'trouver.helper.path_no_ext': ('helper.html#path_no_ext', 'trouver/helper.py'),
                                'trouver.helper.replace_string_by_indices': ('helper.html#replace_string_by_indices', 'trouver/helper.py'),
                                'trouver.helper.text_from_file': ('helper.html#text_from_file', 'trouver/helper.py')},
            'trouver.latex.convert': { 'trouver.latex.convert.NoDocumentNodeError': ( 'latex.convert.html#nodocumentnodeerror',
                                                                                      'trouver/latex/convert.py'),
                                       'trouver.latex.convert.NoDocumentNodeError.__init__': ( 'latex.convert.html#nodocumentnodeerror.__init__',
                                                                                               'trouver/latex/convert.py'),
                                       'trouver.latex.convert._adjust_common_section_titles_in_parts': ( 'latex.convert.html#_adjust_common_section_titles_in_parts',
                                                                                                         'trouver/latex/convert.py'),
                                       'trouver.latex.convert._adjust_common_syntax_to_markdown_in_parts': ( 'latex.convert.html#_adjust_common_syntax_to_markdown_in_parts',
                                                                                                             'trouver/latex/convert.py'),
                                       'trouver.latex.convert._adjusted_title': ( 'latex.convert.html#_adjusted_title',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._append_non_environment_accumulation_to_parts_if_non_empty': ( 'latex.convert.html#_append_non_environment_accumulation_to_parts_if_non_empty',
                                                                                                                             'trouver/latex/convert.py'),
                                       'trouver.latex.convert._argument_detection': ( 'latex.convert.html#_argument_detection',
                                                                                      'trouver/latex/convert.py'),
                                       'trouver.latex.convert._change_counters': ( 'latex.convert.html#_change_counters',
                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._commands_from_newcommand_and_declaremathoperator': ( 'latex.convert.html#_commands_from_newcommand_and_declaremathoperator',
                                                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert._consider_part_to_add': ( 'latex.convert.html#_consider_part_to_add',
                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert._create_note_for_part': ( 'latex.convert.html#_create_note_for_part',
                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert._create_notes_from_parts': ( 'latex.convert.html#_create_notes_from_parts',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert._create_part_or_update': ( 'latex.convert.html#_create_part_or_update',
                                                                                         'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_environment_node': ( 'latex.convert.html#_is_environment_node',
                                                                                       'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_numbered': ( 'latex.convert.html#_is_numbered',
                                                                               'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_numberedwithin': ( 'latex.convert.html#_is_numberedwithin',
                                                                                     'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_section_node': ( 'latex.convert.html#_is_section_node',
                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_subsection_node': ( 'latex.convert.html#_is_subsection_node',
                                                                                      'trouver/latex/convert.py'),
                                       'trouver.latex.convert._make_links_in_index_note_for_section': ( 'latex.convert.html#_make_links_in_index_note_for_section',
                                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert._make_links_in_index_notes': ( 'latex.convert.html#_make_links_in_index_notes',
                                                                                             'trouver/latex/convert.py'),
                                       'trouver.latex.convert._node_is_proof_immediately_following_a_theorem_like_environment': ( 'latex.convert.html#_node_is_proof_immediately_following_a_theorem_like_environment',
                                                                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._node_numbering': ( 'latex.convert.html#_node_numbering',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._node_warrants_own_part': ( 'latex.convert.html#_node_warrants_own_part',
                                                                                          'trouver/latex/convert.py'),
                                       'trouver.latex.convert._numbering_helper': ( 'latex.convert.html#_numbering_helper',
                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert._part_starts_section': ( 'latex.convert.html#_part_starts_section',
                                                                                       'trouver/latex/convert.py'),
                                       'trouver.latex.convert._part_starts_subsection': ( 'latex.convert.html#_part_starts_subsection',
                                                                                          'trouver/latex/convert.py'),
                                       'trouver.latex.convert._process_node': ( 'latex.convert.html#_process_node',
                                                                                'trouver/latex/convert.py'),
                                       'trouver.latex.convert._replace_command': ( 'latex.convert.html#_replace_command',
                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._replace_custom_commands_in_parts': ( 'latex.convert.html#_replace_custom_commands_in_parts',
                                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert._search_counters_by_pattern': ( 'latex.convert.html#_search_counters_by_pattern',
                                                                                              'trouver/latex/convert.py'),
                                       'trouver.latex.convert._search_display_names_by_pattern': ( 'latex.convert.html#_search_display_names_by_pattern',
                                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._section_title': ( 'latex.convert.html#_section_title',
                                                                                 'trouver/latex/convert.py'),
                                       'trouver.latex.convert._setup_all_numberwithins': ( 'latex.convert.html#_setup_all_numberwithins',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert._setup_counters': ( 'latex.convert.html#_setup_counters',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._setup_numberwithins': ( 'latex.convert.html#_setup_numberwithins',
                                                                                       'trouver/latex/convert.py'),
                                       'trouver.latex.convert._title': ('latex.convert.html#_title', 'trouver/latex/convert.py'),
                                       'trouver.latex.convert._title_for_environment_node': ( 'latex.convert.html#_title_for_environment_node',
                                                                                              'trouver/latex/convert.py'),
                                       'trouver.latex.convert._unnumbered_environments': ( 'latex.convert.html#_unnumbered_environments',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert._update_links_to_make': ( 'latex.convert.html#_update_links_to_make',
                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert.adjust_common_syntax_to_markdown': ( 'latex.convert.html#adjust_common_syntax_to_markdown',
                                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert.custom_commands': ( 'latex.convert.html#custom_commands',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.display_names_of_environments': ( 'latex.convert.html#display_names_of_environments',
                                                                                                'trouver/latex/convert.py'),
                                       'trouver.latex.convert.divide_latex_text': ( 'latex.convert.html#divide_latex_text',
                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert.divide_preamble': ( 'latex.convert.html#divide_preamble',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.environment_names_used': ( 'latex.convert.html#environment_names_used',
                                                                                         'trouver/latex/convert.py'),
                                       'trouver.latex.convert.find_document_node': ( 'latex.convert.html#find_document_node',
                                                                                     'trouver/latex/convert.py'),
                                       'trouver.latex.convert.get_node_from_simple_text': ( 'latex.convert.html#get_node_from_simple_text',
                                                                                            'trouver/latex/convert.py'),
                                       'trouver.latex.convert.numbered_newtheorems_counters_in_preamble': ( 'latex.convert.html#numbered_newtheorems_counters_in_preamble',
                                                                                                            'trouver/latex/convert.py'),
                                       'trouver.latex.convert.numberwithins_in_preamble': ( 'latex.convert.html#numberwithins_in_preamble',
                                                                                            'trouver/latex/convert.py'),
                                       'trouver.latex.convert.regex_pattern_detecting_command': ( 'latex.convert.html#regex_pattern_detecting_command',
                                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.remove_comments': ( 'latex.convert.html#remove_comments',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.replace_command_in_text': ( 'latex.convert.html#replace_command_in_text',
                                                                                          'trouver/latex/convert.py'),
                                       'trouver.latex.convert.replace_commands_in_latex_document': ( 'latex.convert.html#replace_commands_in_latex_document',
                                                                                                     'trouver/latex/convert.py'),
                                       'trouver.latex.convert.replace_commands_in_text': ( 'latex.convert.html#replace_commands_in_text',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert.section_and_subsection_titles_from_latex_parts': ( 'latex.convert.html#section_and_subsection_titles_from_latex_parts',
                                                                                                                 'trouver/latex/convert.py'),
                                       'trouver.latex.convert.setup_reference_from_latex_parts': ( 'latex.convert.html#setup_reference_from_latex_parts',
                                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert.swap_numbers_invoked': ( 'latex.convert.html#swap_numbers_invoked',
                                                                                       'trouver/latex/convert.py')},
            'trouver.markdown.markdown.file': { 'trouver.markdown.markdown.file.MarkdownFile': ( 'markdown.markdown.file.html#markdownfile',
                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.__init__': ( 'markdown.markdown.file.html#markdownfile.__init__',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.__str__': ( 'markdown.markdown.file.html#markdownfile.__str__',
                                                                                                         'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._include_previous_line_as_id_text': ( 'markdown.markdown.file.html#markdownfile._include_previous_line_as_id_text',
                                                                                                                                   'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._line_dict': ( 'markdown.markdown.file.html#markdownfile._line_dict',
                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._look_at_start_of_file': ( 'markdown.markdown.file.html#markdownfile._look_at_start_of_file',
                                                                                                                        'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._merge_latex_into_text': ( 'markdown.markdown.file.html#markdownfile._merge_latex_into_text',
                                                                                                                        'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._merge_one_display_math_mode_latex_chunk': ( 'markdown.markdown.file.html#markdownfile._merge_one_display_math_mode_latex_chunk',
                                                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._replace_embedded_links_one_line': ( 'markdown.markdown.file.html#markdownfile._replace_embedded_links_one_line',
                                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._text_of_embedded_link_of_id': ( 'markdown.markdown.file.html#markdownfile._text_of_embedded_link_of_id',
                                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._text_of_embedded_link_of_section': ( 'markdown.markdown.file.html#markdownfile._text_of_embedded_link_of_section',
                                                                                                                                   'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._text_of_lines_of_embedded_links': ( 'markdown.markdown.file.html#markdownfile._text_of_lines_of_embedded_links',
                                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_blank_line_to_end': ( 'markdown.markdown.file.html#markdownfile.add_blank_line_to_end',
                                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_line_in_section': ( 'markdown.markdown.file.html#markdownfile.add_line_in_section',
                                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_line_to_end': ( 'markdown.markdown.file.html#markdownfile.add_line_to_end',
                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_metadata_section': ( 'markdown.markdown.file.html#markdownfile.add_metadata_section',
                                                                                                                      'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_tags': ( 'markdown.markdown.file.html#markdownfile.add_tags',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.clear_all_sections': ( 'markdown.markdown.file.html#markdownfile.clear_all_sections',
                                                                                                                    'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.clear_section': ( 'markdown.markdown.file.html#markdownfile.clear_section',
                                                                                                               'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.copy': ( 'markdown.markdown.file.html#markdownfile.copy',
                                                                                                      'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_file': ( 'markdown.markdown.file.html#markdownfile.from_file',
                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_list': ( 'markdown.markdown.file.html#markdownfile.from_list',
                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_string': ( 'markdown.markdown.file.html#markdownfile.from_string',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_vault_note': ( 'markdown.markdown.file.html#markdownfile.from_vault_note',
                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings': ( 'markdown.markdown.file.html#markdownfile.get_headings',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings_and_text': ( 'markdown.markdown.file.html#markdownfile.get_headings_and_text',
                                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings_by_line_number': ( 'markdown.markdown.file.html#markdownfile.get_headings_by_line_number',
                                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings_tree': ( 'markdown.markdown.file.html#markdownfile.get_headings_tree',
                                                                                                                   'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_line_number_of_heading': ( 'markdown.markdown.file.html#markdownfile.get_line_number_of_heading',
                                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_line_numbers_under_heading': ( 'markdown.markdown.file.html#markdownfile.get_line_numbers_under_heading',
                                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.has_metadata': ( 'markdown.markdown.file.html#markdownfile.has_metadata',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.has_tag': ( 'markdown.markdown.file.html#markdownfile.has_tag',
                                                                                                         'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.insert_line': ( 'markdown.markdown.file.html#markdownfile.insert_line',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.merge_display_math_mode': ( 'markdown.markdown.file.html#markdownfile.merge_display_math_mode',
                                                                                                                         'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.merge_display_math_mode_into_preceding_text': ( 'markdown.markdown.file.html#markdownfile.merge_display_math_mode_into_preceding_text',
                                                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.metadata': ( 'markdown.markdown.file.html#markdownfile.metadata',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.metadata_lines': ( 'markdown.markdown.file.html#markdownfile.metadata_lines',
                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.parts_of_id': ( 'markdown.markdown.file.html#markdownfile.parts_of_id',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.pop_line': ( 'markdown.markdown.file.html#markdownfile.pop_line',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_double_blank_lines': ( 'markdown.markdown.file.html#markdownfile.remove_double_blank_lines',
                                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_footnotes_to_embedded_links': ( 'markdown.markdown.file.html#markdownfile.remove_footnotes_to_embedded_links',
                                                                                                                                    'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_headers': ( 'markdown.markdown.file.html#markdownfile.remove_headers',
                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_in_line_tags': ( 'markdown.markdown.file.html#markdownfile.remove_in_line_tags',
                                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_line': ( 'markdown.markdown.file.html#markdownfile.remove_line',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_lines': ( 'markdown.markdown.file.html#markdownfile.remove_lines',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_metadata': ( 'markdown.markdown.file.html#markdownfile.remove_metadata',
                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_section': ( 'markdown.markdown.file.html#markdownfile.remove_section',
                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_tags': ( 'markdown.markdown.file.html#markdownfile.remove_tags',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_auto_tags_with_regular_tags': ( 'markdown.markdown.file.html#markdownfile.replace_auto_tags_with_regular_tags',
                                                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_embedded_links_with_text': ( 'markdown.markdown.file.html#markdownfile.replace_embedded_links_with_text',
                                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_links_with_display_text': ( 'markdown.markdown.file.html#markdownfile.replace_links_with_display_text',
                                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_metadata': ( 'markdown.markdown.file.html#markdownfile.replace_metadata',
                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.text_of_lines': ( 'markdown.markdown.file.html#markdownfile.text_of_lines',
                                                                                                               'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.write': ( 'markdown.markdown.file.html#markdownfile.write',
                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownLineEnum': ( 'markdown.markdown.file.html#markdownlineenum',
                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._add_line_for_list': ( 'markdown.markdown.file.html#_add_line_for_list',
                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._enquote': ( 'markdown.markdown.file.html#_enquote',
                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._line_end_in_line_latex': ( 'markdown.markdown.file.html#_line_end_in_line_latex',
                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._line_start_and_end_in_line_latex': ( 'markdown.markdown.file.html#_line_start_and_end_in_line_latex',
                                                                                                                      'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._line_start_in_line_latex': ( 'markdown.markdown.file.html#_line_start_in_line_latex',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._remove_text_id': ( 'markdown.markdown.file.html#_remove_text_id',
                                                                                                    'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.dict_to_metadata_lines': ( 'markdown.markdown.file.html#dict_to_metadata_lines',
                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.find_front_matter_meta_in_markdown_text': ( 'markdown.markdown.file.html#find_front_matter_meta_in_markdown_text',
                                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.replace_embedded_links_with_text': ( 'markdown.markdown.file.html#replace_embedded_links_with_text',
                                                                                                                     'trouver/markdown/markdown/file.py')},
            'trouver.markdown.markdown.heading': { 'trouver.markdown.markdown.heading.heading_level': ( 'markdown.markdown.heading.html#heading_level',
                                                                                                        'trouver/markdown/markdown/heading.py'),
                                                   'trouver.markdown.markdown.heading.heading_title': ( 'markdown.markdown.heading.html#heading_title',
                                                                                                        'trouver/markdown/markdown/heading.py')},
            'trouver.markdown.obsidian.footnotes': { 'trouver.markdown.obsidian.footnotes.embedded_note_of_footnote': ( 'markdown.obsidian.footnotes.html#embedded_note_of_footnote',
                                                                                                                        'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.find_footnote_descriptions_in_markdown_text': ( 'markdown.obsidian.footnotes.html#find_footnote_descriptions_in_markdown_text',
                                                                                                                                          'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.find_footnote_mentions_in_markdown_text': ( 'markdown.obsidian.footnotes.html#find_footnote_mentions_in_markdown_text',
                                                                                                                                      'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.footnote_is_simple_embedded_note': ( 'markdown.obsidian.footnotes.html#footnote_is_simple_embedded_note',
                                                                                                                               'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.remove_footnote_mentions_in_markdown_text': ( 'markdown.obsidian.footnotes.html#remove_footnote_mentions_in_markdown_text',
                                                                                                                                        'trouver/markdown/obsidian/footnotes.py')},
            'trouver.markdown.obsidian.links': { 'trouver.markdown.obsidian.links.LinkFormatError': ( 'markdown.obsidian.links.html#linkformaterror',
                                                                                                      'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.LinkFormatError.__init__': ( 'markdown.obsidian.links.html#linkformaterror.__init__',
                                                                                                               'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.LinkType': ( 'markdown.obsidian.links.html#linktype',
                                                                                               'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink': ( 'markdown.obsidian.links.html#obsidianlink',
                                                                                                   'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.__init__': ( 'markdown.obsidian.links.html#obsidianlink.__init__',
                                                                                                            'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.__str__': ( 'markdown.obsidian.links.html#obsidianlink.__str__',
                                                                                                           'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink._parse_text_as_wikilink': ( 'markdown.obsidian.links.html#obsidianlink._parse_text_as_wikilink',
                                                                                                                           'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.convert_link_type': ( 'markdown.obsidian.links.html#obsidianlink.convert_link_type',
                                                                                                                     'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.displayed_text': ( 'markdown.obsidian.links.html#obsidianlink.displayed_text',
                                                                                                                  'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.from_text': ( 'markdown.obsidian.links.html#obsidianlink.from_text',
                                                                                                             'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.is_abstract': ( 'markdown.obsidian.links.html#obsidianlink.is_abstract',
                                                                                                               'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.to_regex': ( 'markdown.obsidian.links.html#obsidianlink.to_regex',
                                                                                                            'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.to_string': ( 'markdown.obsidian.links.html#obsidianlink.to_string',
                                                                                                             'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links._custom_text_for_links_in_text': ( 'markdown.obsidian.links.html#_custom_text_for_links_in_text',
                                                                                                                     'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links._do_not_remove_link': ( 'markdown.obsidian.links.html#_do_not_remove_link',
                                                                                                          'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.all_custom_text_for_links_in_vault': ( 'markdown.obsidian.links.html#all_custom_text_for_links_in_vault',
                                                                                                                         'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.all_links_in_vault': ( 'markdown.obsidian.links.html#all_links_in_vault',
                                                                                                         'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.find_links_in_markdown_text': ( 'markdown.obsidian.links.html#find_links_in_markdown_text',
                                                                                                                  'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.links_from_text': ( 'markdown.obsidian.links.html#links_from_text',
                                                                                                      'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.remove_links_from_text': ( 'markdown.obsidian.links.html#remove_links_from_text',
                                                                                                             'trouver/markdown/obsidian/links.py')},
            'trouver.markdown.obsidian.personal.authors': { 'trouver.markdown.obsidian.personal.authors.find_author_file': ( 'markdown.obisidian.personal.authors.html#find_author_file',
                                                                                                                             'trouver/markdown/obsidian/personal/authors.py'),
                                                            'trouver.markdown.obsidian.personal.authors.just_last_name': ( 'markdown.obisidian.personal.authors.html#just_last_name',
                                                                                                                           'trouver/markdown/obsidian/personal/authors.py')},
            'trouver.markdown.obsidian.personal.index_notes': { 'trouver.markdown.obsidian.personal.index_notes._move_notes_under_heading': ( 'markdown.obsidian.personal.index_notes.html#_move_notes_under_heading',
                                                                                                                                              'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.add_link_in_index_note_after_note_link': ( 'markdown.obsidian.personal.index_notes.html#add_link_in_index_note_after_note_link',
                                                                                                                                                           'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.convert_heading_to_folder_name': ( 'markdown.obsidian.personal.index_notes.html#convert_heading_to_folder_name',
                                                                                                                                                   'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.convert_title_to_folder_name': ( 'markdown.obsidian.personal.index_notes.html#convert_title_to_folder_name',
                                                                                                                                                 'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.correspond_headings_with_folder': ( 'markdown.obsidian.personal.index_notes.html#correspond_headings_with_folder',
                                                                                                                                                    'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.get_alphanumeric': ( 'markdown.obsidian.personal.index_notes.html#get_alphanumeric',
                                                                                                                                     'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.get_notes_from_index_note': ( 'markdown.obsidian.personal.index_notes.html#get_notes_from_index_note',
                                                                                                                                              'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.information_notes_linked_in_index_note': ( 'markdown.obsidian.personal.index_notes.html#information_notes_linked_in_index_note',
                                                                                                                                                           'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.make_folders_from_index_note_headers': ( 'markdown.obsidian.personal.index_notes.html#make_folders_from_index_note_headers',
                                                                                                                                                         'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.move_information_notes_to_correct_folder': ( 'markdown.obsidian.personal.index_notes.html#move_information_notes_to_correct_folder',
                                                                                                                                                             'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.move_information_notes_to_correct_folder_for_all_indices': ( 'markdown.obsidian.personal.index_notes.html#move_information_notes_to_correct_folder_for_all_indices',
                                                                                                                                                                             'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.subsection_folders': ( 'markdown.obsidian.personal.index_notes.html#subsection_folders',
                                                                                                                                       'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.subsections_listed_in_index_note': ( 'markdown.obsidian.personal.index_notes.html#subsections_listed_in_index_note',
                                                                                                                                                     'trouver/markdown/obsidian/personal/index_notes.py')},
            'trouver.markdown.obsidian.personal.information_notes': { 'trouver.markdown.obsidian.personal.information_notes._index_note_in_parent_directory': ( 'markdown.obsidian.personal.information_notes.html#_index_note_in_parent_directory',
                                                                                                                                                                'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes._index_note_in_same_directory': ( 'markdown.obsidian.personal.information_notes.html#_index_note_in_same_directory',
                                                                                                                                                              'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes._note_is_not_linked_in_index_note': ( 'markdown.obsidian.personal.information_notes.html#_note_is_not_linked_in_index_note',
                                                                                                                                                                  'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.bulleted_links_of_type_in_section': ( 'markdown.obsidian.personal.information_notes.html#bulleted_links_of_type_in_section',
                                                                                                                                                                  'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.citation_location_string': ( 'markdown.obsidian.personal.information_notes.html#citation_location_string',
                                                                                                                                                         'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.create_generic_info_notes_and_link_to_index': ( 'markdown.obsidian.personal.information_notes.html#create_generic_info_notes_and_link_to_index',
                                                                                                                                                                            'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.create_info_notes_and_link_to_index': ( 'markdown.obsidian.personal.information_notes.html#create_info_notes_and_link_to_index',
                                                                                                                                                                    'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.fill_info_note_with_template': ( 'markdown.obsidian.personal.information_notes.html#fill_info_note_with_template',
                                                                                                                                                             'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.index_note_of_a_directory': ( 'markdown.obsidian.personal.information_notes.html#index_note_of_a_directory',
                                                                                                                                                          'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.index_note_of_note': ( 'markdown.obsidian.personal.information_notes.html#index_note_of_note',
                                                                                                                                                   'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.link_info_notes_to_index': ( 'markdown.obsidian.personal.information_notes.html#link_info_notes_to_index',
                                                                                                                                                         'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.links_to_common_terms_notes_in_see_also_section': ( 'markdown.obsidian.personal.information_notes.html#links_to_common_terms_notes_in_see_also_section',
                                                                                                                                                                                'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.main_content': ( 'markdown.obsidian.personal.information_notes.html#main_content',
                                                                                                                                             'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.reference_notes_in_references_section_of_information_note': ( 'markdown.obsidian.personal.information_notes.html#reference_notes_in_references_section_of_information_note',
                                                                                                                                                                                          'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.reference_of_information_note': ( 'markdown.obsidian.personal.information_notes.html#reference_of_information_note',
                                                                                                                                                              'trouver/markdown/obsidian/personal/information_notes.py')},
            'trouver.markdown.obsidian.personal.machine_learning.database_update': { 'trouver.markdown.obsidian.personal.machine_learning.database_update.append_to_database': ( 'markdown.obsidian.personal.machine_learninig.database_update.html#append_to_database',
                                                                                                                                                                                 'trouver/markdown/obsidian/personal/machine_learning/database_update.py'),
                                                                                     'trouver.markdown.obsidian.personal.machine_learning.database_update.max_ID': ( 'markdown.obsidian.personal.machine_learninig.database_update.html#max_id',
                                                                                                                                                                     'trouver/markdown/obsidian/personal/machine_learning/database_update.py')},
            'trouver.markdown.obsidian.personal.machine_learning.definition_identification': { 'trouver.markdown.obsidian.personal.machine_learning.definition_identification.definition_identification_data_from_note': ( 'markdown.obsidian.personal.machine_learning.definition_identification.html#definition_identification_data_from_note',
                                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/definition_identification.py'),
                                                                                               'trouver.markdown.obsidian.personal.machine_learning.definition_identification.definitions_in_text': ( 'markdown.obsidian.personal.machine_learning.definition_identification.html#definitions_in_text',
                                                                                                                                                                                                      'trouver/markdown/obsidian/personal/machine_learning/definition_identification.py'),
                                                                                               'trouver.markdown.obsidian.personal.machine_learning.definition_identification.gather_definition_identification_data': ( 'markdown.obsidian.personal.machine_learning.definition_identification.html#gather_definition_identification_data',
                                                                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/definition_identification.py')},
            'trouver.markdown.obsidian.personal.machine_learning.information_note_types': { 'trouver.markdown.obsidian.personal.machine_learning.information_note_types._append_single_predicted_tag': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_append_single_predicted_tag',
                                                                                                                                                                                                         'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._auto_prediction': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_auto_prediction',
                                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._change_label_tags_for_single_note': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_change_label_tags_for_single_note',
                                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._has_any_label_tags': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_has_any_label_tags',
                                                                                                                                                                                                'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._has_exactly_predicted_tags': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_has_exactly_predicted_tags',
                                                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.append_to_information_note_type_database': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#append_to_information_note_type_database',
                                                                                                                                                                                                                     'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.automatically_add_note_type_tags': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#automatically_add_note_type_tags',
                                                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.convert_auto_tags_to_regular_tags_in_notes': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#convert_auto_tags_to_regular_tags_in_notes',
                                                                                                                                                                                                                       'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.gather_information_note_types': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#gather_information_note_types',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.note_is_labeled_with_tag': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#note_is_labeled_with_tag',
                                                                                                                                                                                                     'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.note_labels': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#note_labels',
                                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.predict_note_types': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#predict_note_types',
                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.predict_text_types': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#predict_text_types',
                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notation': { 'trouver.markdown.obsidian.personal.machine_learning.notation._get_notation_indices_to_add': ( 'markdown.obsidian.personal.machine_learning.notation.html#_get_notation_indices_to_add',
                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation._notation_data_with_indices': ( 'markdown.obsidian.personal.machine_learning.notation.html#_notation_data_with_indices',
                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.add_one_double_asts_to_line': ( 'markdown.obsidian.personal.machine_learning.notation.html#add_one_double_asts_to_line',
                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.append_notation_data_to_database': ( 'markdown.obsidian.personal.machine_learning.notation.html#append_notation_data_to_database',
                                                                                                                                                                                 'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.automatically_mark_notations': ( 'markdown.obsidian.personal.machine_learning.notation.html#automatically_mark_notations',
                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.notation_data_from_note': ( 'markdown.obsidian.personal.machine_learning.notation.html#notation_data_from_note',
                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.notation_data_from_text': ( 'markdown.obsidian.personal.machine_learning.notation.html#notation_data_from_text',
                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/notation.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notation_identification': { 'trouver.markdown.obsidian.personal.machine_learning.notation_identification._get_notation_indices_to_add': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#_get_notation_indices_to_add',
                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification._notation_data_with_indices': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#_notation_data_with_indices',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.add_one_double_asts_to_line': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#add_one_double_asts_to_line',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.append_notation_data_to_database': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#append_notation_data_to_database',
                                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.automatically_mark_notations': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#automatically_mark_notations',
                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.notation_data_from_note': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#notation_data_from_note',
                                                                                                                                                                                                      'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.notation_data_from_text': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#notation_data_from_text',
                                                                                                                                                                                                      'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization': { 'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._escape_latex_in_original_in_metadata': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_escape_latex_in_original_in_metadata',
                                                                                                                                                                                                                  'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._get_summary': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_get_summary',
                                                                                                                                                                                         'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._metadata_dict_has_auto_notation_summary_tag': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_metadata_dict_has_auto_notation_summary_tag',
                                                                                                                                                                                                                         'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._notation_has_been_summarized': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_notation_has_been_summarized',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._notation_note_has_auto_summary_tag': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_notation_note_has_auto_summary_tag',
                                                                                                                                                                                                                'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._write_summary_to_notation_note': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_write_summary_to_notation_note',
                                                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.append_column_for_single_text': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#append_column_for_single_text',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.append_summary_to_notation_note': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#append_summary_to_notation_note',
                                                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.append_to_notation_note_summarization_database': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#append_to_notation_note_summarization_database',
                                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.fix_summary_formatting': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#fix_summary_formatting',
                                                                                                                                                                                                   'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.gather_notation_note_summaries': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#gather_notation_note_summaries',
                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.get_latex_in_original_from_parsed_notation_note_data': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#get_latex_in_original_from_parsed_notation_note_data',
                                                                                                                                                                                                                                 'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.notation_summarization_data_from_note': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#notation_summarization_data_from_note',
                                                                                                                                                                                                                  'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.single_input_for_notation_summarization': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#single_input_for_notation_summarization',
                                                                                                                                                                                                                    'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.summarize_notation': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#summarize_notation',
                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notations': {},
            'trouver.markdown.obsidian.personal.notation': { 'trouver.markdown.obsidian.personal.notation._build_replacables_from_groups': ( 'markdown.obsidian.personal.notation.html#_build_replacables_from_groups',
                                                                                                                                             'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._divide_bulleted_list_mf_at_end': ( 'markdown.obsidian.personal.notation.html#_divide_bulleted_list_mf_at_end',
                                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._full_notation_string': ( 'markdown.obsidian.personal.notation.html#_full_notation_string',
                                                                                                                                    'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._get_notation_string': ( 'markdown.obsidian.personal.notation.html#_get_notation_string',
                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._latex_in_original_from_notat_notes_to_main_note': ( 'markdown.obsidian.personal.notation.html#_latex_in_original_from_notat_notes_to_main_note',
                                                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._latex_in_original_in_notat': ( 'markdown.obsidian.personal.notation.html#_latex_in_original_in_notat',
                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._look_into_node': ( 'markdown.obsidian.personal.notation.html#_look_into_node',
                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._macro_is_actually_placeholder': ( 'markdown.obsidian.personal.notation.html#_macro_is_actually_placeholder',
                                                                                                                                             'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._main_of_notation_from_text': ( 'markdown.obsidian.personal.notation.html#_main_of_notation_from_text',
                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._make_new_notes_from_sifted_double_asts': ( 'markdown.obsidian.personal.notation.html#_make_new_notes_from_sifted_double_asts',
                                                                                                                                                      'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._part_is_unordered_list_and_is_of_markdownstyle_link': ( 'markdown.obsidian.personal.notation.html#_part_is_unordered_list_and_is_of_markdownstyle_link',
                                                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._raw_notation': ( 'markdown.obsidian.personal.notation.html#_raw_notation',
                                                                                                                            'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._remove_the_notation_str_and_denotes_in_main_mf': ( 'markdown.obsidian.personal.notation.html#_remove_the_notation_str_and_denotes_in_main_mf',
                                                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.add_missing_notation_links_to_information_notes': ( 'markdown.obsidian.personal.notation.html#add_missing_notation_links_to_information_notes',
                                                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.add_notation_note_to_see_also': ( 'markdown.obsidian.personal.notation.html#add_notation_note_to_see_also',
                                                                                                                                            'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.index_notation_note_formatted_entry': ( 'markdown.obsidian.personal.notation.html#index_notation_note_formatted_entry',
                                                                                                                                                  'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.main_of_notation': ( 'markdown.obsidian.personal.notation.html#main_of_notation',
                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.make_a_notation_note': ( 'markdown.obsidian.personal.notation.html#make_a_notation_note',
                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.make_notation_notes_from_double_asts': ( 'markdown.obsidian.personal.notation.html#make_notation_notes_from_double_asts',
                                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_in_note': ( 'markdown.obsidian.personal.notation.html#notation_in_note',
                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_note_is_linked_in_see_also_section': ( 'markdown.obsidian.personal.notation.html#notation_note_is_linked_in_see_also_section',
                                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_notes_linked_in_see_also_section': ( 'markdown.obsidian.personal.notation.html#notation_notes_linked_in_see_also_section',
                                                                                                                                                        'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_str_in_a_standard_information_note': ( 'markdown.obsidian.personal.notation.html#notation_str_in_a_standard_information_note',
                                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notations_and_main_notes': ( 'markdown.obsidian.personal.notation.html#notations_and_main_notes',
                                                                                                                                       'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notations_to_add_in_index': ( 'markdown.obsidian.personal.notation.html#notations_to_add_in_index',
                                                                                                                                        'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.parse_notation_note': ( 'markdown.obsidian.personal.notation.html#parse_notation_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.regex_from_latex': ( 'markdown.obsidian.personal.notation.html#regex_from_latex',
                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.regex_from_notation_note': ( 'markdown.obsidian.personal.notation.html#regex_from_notation_note',
                                                                                                                                       'trouver/markdown/obsidian/personal/notation.py')},
            'trouver.markdown.obsidian.personal.note_processing': { 'trouver.markdown.obsidian.personal.note_processing.process_standard_information_note': ( 'markdown.obsidian.personal.note_processing.html#process_standard_information_note',
                                                                                                                                                              'trouver/markdown/obsidian/personal/note_processing.py'),
                                                                    'trouver.markdown.obsidian.personal.note_processing.remove_double_asterisks_in_markdown_file': ( 'markdown.obsidian.personal.note_processing.html#remove_double_asterisks_in_markdown_file',
                                                                                                                                                                     'trouver/markdown/obsidian/personal/note_processing.py')},
            'trouver.markdown.obsidian.personal.note_type': { 'trouver.markdown.obsidian.personal.note_type.NoteTypeError': ( 'markdown.obsidian.personal.note_type.html#notetypeerror',
                                                                                                                              'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.NoteTypeError.__init__': ( 'markdown.obsidian.personal.note_type.html#notetypeerror.__init__',
                                                                                                                                       'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.PersonalNoteTypeEnum': ( 'markdown.obsidian.personal.note_type.html#personalnotetypeenum',
                                                                                                                                     'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.PersonalNoteTypeEnum.type_function': ( 'markdown.obsidian.personal.note_type.html#personalnotetypeenum.type_function',
                                                                                                                                                   'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_common_terms_note': ( 'markdown.obsidian.personal.note_type.html#_is_common_terms_note',
                                                                                                                                      'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_equation_note': ( 'markdown.obsidian.personal.note_type.html#_is_equation_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_glossary_note': ( 'markdown.obsidian.personal.note_type.html#_is_glossary_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_index_note': ( 'markdown.obsidian.personal.note_type.html#_is_index_note',
                                                                                                                               'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_index_of_notation_note': ( 'markdown.obsidian.personal.note_type.html#_is_index_of_notation_note',
                                                                                                                                           'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_mathematician_note': ( 'markdown.obsidian.personal.note_type.html#_is_mathematician_note',
                                                                                                                                       'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_module_information_note': ( 'markdown.obsidian.personal.note_type.html#_is_module_information_note',
                                                                                                                                            'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_notation_note': ( 'markdown.obsidian.personal.note_type.html#_is_notation_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_notes_note': ( 'markdown.obsidian.personal.note_type.html#_is_notes_note',
                                                                                                                               'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_premise_note': ( 'markdown.obsidian.personal.note_type.html#_is_premise_note',
                                                                                                                                 'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_reference_note': ( 'markdown.obsidian.personal.note_type.html#_is_reference_note',
                                                                                                                                   'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_standard_information_note': ( 'markdown.obsidian.personal.note_type.html#_is_standard_information_note',
                                                                                                                                              'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_tag_note': ( 'markdown.obsidian.personal.note_type.html#_is_tag_note',
                                                                                                                             'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_template_note': ( 'markdown.obsidian.personal.note_type.html#_is_template_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.assert_note_is_of_type': ( 'markdown.obsidian.personal.note_type.html#assert_note_is_of_type',
                                                                                                                                       'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.note_is_of_type': ( 'markdown.obsidian.personal.note_type.html#note_is_of_type',
                                                                                                                                'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.type_of_note': ( 'markdown.obsidian.personal.note_type.html#type_of_note',
                                                                                                                             'trouver/markdown/obsidian/personal/note_type.py')},
            'trouver.markdown.obsidian.personal.notes': { 'trouver.markdown.obsidian.personal.notes.note_has_tag': ( 'markdown.obsidian.personal.notes.html#note_has_tag',
                                                                                                                     'trouver/markdown/obsidian/personal/notes.py'),
                                                          'trouver.markdown.obsidian.personal.notes.notes_linked_in_note': ( 'markdown.obsidian.personal.notes.html#notes_linked_in_note',
                                                                                                                             'trouver/markdown/obsidian/personal/notes.py'),
                                                          'trouver.markdown.obsidian.personal.notes.notes_linked_in_notes_linked_in_note': ( 'markdown.obsidian.personal.notes.html#notes_linked_in_notes_linked_in_note',
                                                                                                                                             'trouver/markdown/obsidian/personal/notes.py')},
            'trouver.markdown.obsidian.personal.reference': { 'trouver.markdown.obsidian.personal.reference._chapter_titles': ( 'markdown.obsidian.personal.reference.html#_chapter_titles',
                                                                                                                                'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._create_template_note_at': ( 'markdown.obsidian.personal.reference.html#_create_template_note_at',
                                                                                                                                         'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._files_in_reference_folder_via_glob': ( 'markdown.obsidian.personal.reference.html#_files_in_reference_folder_via_glob',
                                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._files_in_reference_folder_via_index_notes': ( 'markdown.obsidian.personal.reference.html#_files_in_reference_folder_via_index_notes',
                                                                                                                                                           'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_chapter_folders_and_indices': ( 'markdown.obsidian.personal.reference.html#_make_chapter_folders_and_indices',
                                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_glossary_file': ( 'markdown.obsidian.personal.reference.html#_make_glossary_file',
                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_index_file': ( 'markdown.obsidian.personal.reference.html#_make_index_file',
                                                                                                                                 'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_notation_index_file': ( 'markdown.obsidian.personal.reference.html#_make_notation_index_file',
                                                                                                                                          'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_reference_file': ( 'markdown.obsidian.personal.reference.html#_make_reference_file',
                                                                                                                                     'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_reference_folder': ( 'markdown.obsidian.personal.reference.html#_make_reference_folder',
                                                                                                                                       'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_single_chapter_folders': ( 'markdown.obsidian.personal.reference.html#_make_single_chapter_folders',
                                                                                                                                             'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_single_chapter_index': ( 'markdown.obsidian.personal.reference.html#_make_single_chapter_index',
                                                                                                                                           'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_temp_folder': ( 'markdown.obsidian.personal.reference.html#_make_temp_folder',
                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_template_file': ( 'markdown.obsidian.personal.reference.html#_make_template_file',
                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._manifest_template_file': ( 'markdown.obsidian.personal.reference.html#_manifest_template_file',
                                                                                                                                        'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._modify_fast_link_edit_and_templates': ( 'markdown.obsidian.personal.reference.html#_modify_fast_link_edit_and_templates',
                                                                                                                                                     'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._obsidian_vault_plugin_configs_file': ( 'markdown.obsidian.personal.reference.html#_obsidian_vault_plugin_configs_file',
                                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.copy_obsidian_vault_configs': ( 'markdown.obsidian.personal.reference.html#copy_obsidian_vault_configs',
                                                                                                                                            'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.copy_obsidian_vault_configs_with_nice_modifications': ( 'markdown.obsidian.personal.reference.html#copy_obsidian_vault_configs_with_nice_modifications',
                                                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.delete_reference_folder': ( 'markdown.obsidian.personal.reference.html#delete_reference_folder',
                                                                                                                                        'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.files_in_reference_folder': ( 'markdown.obsidian.personal.reference.html#files_in_reference_folder',
                                                                                                                                          'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.get_index_notes_from_index_note': ( 'markdown.obsidian.personal.reference.html#get_index_notes_from_index_note',
                                                                                                                                                'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.get_index_notes_from_subdirectory': ( 'markdown.obsidian.personal.reference.html#get_index_notes_from_subdirectory',
                                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.get_obsidian_vault_plugin_configs': ( 'markdown.obsidian.personal.reference.html#get_obsidian_vault_plugin_configs',
                                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.index_note_for_reference': ( 'markdown.obsidian.personal.reference.html#index_note_for_reference',
                                                                                                                                         'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.modify_obsidian_vault_plugin_configs': ( 'markdown.obsidian.personal.reference.html#modify_obsidian_vault_plugin_configs',
                                                                                                                                                     'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.reference_directory': ( 'markdown.obsidian.personal.reference.html#reference_directory',
                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.reference_folders_in_vault': ( 'markdown.obsidian.personal.reference.html#reference_folders_in_vault',
                                                                                                                                           'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.setup_folder_for_new_reference': ( 'markdown.obsidian.personal.reference.html#setup_folder_for_new_reference',
                                                                                                                                               'trouver/markdown/obsidian/personal/reference.py')},
            'trouver.markdown.obsidian.tags': { 'trouver.markdown.obsidian.tags.strip_auto_from_tag': ( 'markdown.obsidian.tags.html#strip_auto_from_tag',
                                                                                                        'trouver/markdown/obsidian/tags.py'),
                                                'trouver.markdown.obsidian.tags.tag_is_auto_tag': ( 'markdown.obsidian.tags.html#tag_is_auto_tag',
                                                                                                    'trouver/markdown/obsidian/tags.py'),
                                                'trouver.markdown.obsidian.tags.tag_without_hashtag': ( 'markdown.obsidian.tags.html#tag_without_hashtag',
                                                                                                        'trouver/markdown/obsidian/tags.py'),
                                                'trouver.markdown.obsidian.tags.to_tag_str': ( 'markdown.obsidian.tags.html#to_tag_str',
                                                                                               'trouver/markdown/obsidian/tags.py')},
            'trouver.markdown.obsidian.vault': { 'trouver.markdown.obsidian.vault.NoteDoesNotExistError': ( 'markdown.obsidian.vault.html#notedoesnotexisterror',
                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteDoesNotExistError.__init__': ( 'markdown.obsidian.vault.html#notedoesnotexisterror.__init__',
                                                                                                                     'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteDoesNotExistError.from_note_name': ( 'markdown.obsidian.vault.html#notedoesnotexisterror.from_note_name',
                                                                                                                           'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteNotUniqueError': ( 'markdown.obsidian.vault.html#notenotuniqueerror',
                                                                                                         'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteNotUniqueError.__init__': ( 'markdown.obsidian.vault.html#notenotuniqueerror.__init__',
                                                                                                                  'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteNotUniqueError.from_note_names': ( 'markdown.obsidian.vault.html#notenotuniqueerror.from_note_names',
                                                                                                                         'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NotePathIsNotIdentifiedError': ( 'markdown.obsidian.vault.html#notepathisnotidentifiederror',
                                                                                                                   'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NotePathIsNotIdentifiedError.__init__': ( 'markdown.obsidian.vault.html#notepathisnotidentifiederror.__init__',
                                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NotePathIsNotIdentifiedError.from_note': ( 'markdown.obsidian.vault.html#notepathisnotidentifiederror.from_note',
                                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote': ( 'markdown.obsidian.vault.html#vaultnote',
                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.__init__': ( 'markdown.obsidian.vault.html#vaultnote.__init__',
                                                                                                         'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._add_single_entry_to_cache': ( 'markdown.obsidian.vault.html#vaultnote._add_single_entry_to_cache',
                                                                                                                           'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._check_if_cache_needs_to_update': ( 'markdown.obsidian.vault.html#vaultnote._check_if_cache_needs_to_update',
                                                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._check_name_exists_and_unique_in_vault_cache': ( 'markdown.obsidian.vault.html#vaultnote._check_name_exists_and_unique_in_vault_cache',
                                                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._get_from_cache': ( 'markdown.obsidian.vault.html#vaultnote._get_from_cache',
                                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._identify_rel_path': ( 'markdown.obsidian.vault.html#vaultnote._identify_rel_path',
                                                                                                                   'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._remove_single_entry_from_cache': ( 'markdown.obsidian.vault.html#vaultnote._remove_single_entry_from_cache',
                                                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.clear_cache': ( 'markdown.obsidian.vault.html#vaultnote.clear_cache',
                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.create': ( 'markdown.obsidian.vault.html#vaultnote.create',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.delete': ( 'markdown.obsidian.vault.html#vaultnote.delete',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.directory': ( 'markdown.obsidian.vault.html#vaultnote.directory',
                                                                                                          'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.exists': ( 'markdown.obsidian.vault.html#vaultnote.exists',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.identify_rel_path': ( 'markdown.obsidian.vault.html#vaultnote.identify_rel_path',
                                                                                                                  'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.move_to': ( 'markdown.obsidian.vault.html#vaultnote.move_to',
                                                                                                        'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.move_to_folder': ( 'markdown.obsidian.vault.html#vaultnote.move_to_folder',
                                                                                                               'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.obsidian_identifier': ( 'markdown.obsidian.vault.html#vaultnote.obsidian_identifier',
                                                                                                                    'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.path': ( 'markdown.obsidian.vault.html#vaultnote.path',
                                                                                                     'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.rel_path_identified': ( 'markdown.obsidian.vault.html#vaultnote.rel_path_identified',
                                                                                                                    'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.text': ( 'markdown.obsidian.vault.html#vaultnote.text',
                                                                                                     'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.unique_name': ( 'markdown.obsidian.vault.html#vaultnote.unique_name',
                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.update_cache': ( 'markdown.obsidian.vault.html#vaultnote.update_cache',
                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.all_note_paths_by_name': ( 'markdown.obsidian.vault.html#all_note_paths_by_name',
                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.all_paths_to_notes_in_vault': ( 'markdown.obsidian.vault.html#all_paths_to_notes_in_vault',
                                                                                                                  'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.note_name_from_path': ( 'markdown.obsidian.vault.html#note_name_from_path',
                                                                                                          'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.note_name_unique': ( 'markdown.obsidian.vault.html#note_name_unique',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.note_path_by_name': ( 'markdown.obsidian.vault.html#note_path_by_name',
                                                                                                        'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.path_to_obs_id': ( 'markdown.obsidian.vault.html#path_to_obs_id',
                                                                                                     'trouver/markdown/obsidian/vault.py')}}}
