# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../../../nbs/17_markdown.obsidian.personal.machine_learning.__init__.ipynb.

# %% auto 0
__all__ = []

# %% ../../../../../nbs/17_markdown.obsidian.personal.machine_learning.__init__.ipynb 2
from bisect import bisect_left, bisect_right
from os import PathLike
from random import sample, choices
import re
from typing import Union

# from fastai.text.all import *
import pandas as pd

from .....helper import latex_indices, find_regex_in_text, current_time_formatted_to_minutes
from ....markdown.file import MarkdownFile
from trouver.markdown.obsidian.footnotes import (
    remove_footnote_mentions_in_markdown_text
)
# TODO: import the rest
